using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

using Emgu.CV;
using Emgu.CV.Structure;

namespace TestProject
{
    /// <summary>
    ///  
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Import
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Ŀ ġ ϱ - GetCursorPos(point)

        /// <summary>
        /// Ŀ ġ ϱ
        /// </summary>
        /// <param name="point">ġ</param>
        /// <returns>ó </returns>
        [DllImport("user32.dll")]
        private static extern bool GetCursorPos(out Point point);

        #endregion
        #region Ŀ ġ ϱ - SetCursorPos(x, y)

        /// <summary>
        /// Ŀ ġ ϱ
        /// </summary>
        /// <param name="x">X ǥ</param>
        /// <param name="y">Y ǥ</param>
        /// <returns>ó </returns>
        [DllImport("user32.dll")]
        private static extern bool SetCursorPos(int x, int y);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// ĸó
        /// </summary>
        private Capture capture;

        /// <summary>
        /// з
        /// </summary>
        private CascadeClassifier classifier;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - MainForm()

        /// <summary>
        /// 
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.classifier = new CascadeClassifier("haarcascade_frontalface_alt2.xml");

            if(this.capture == null)
            {
                try
                {
                    this.capture = new Capture();
                }
                catch(Exception exception)
                {
                    MessageBox.Show(exception.Message);

                    return;
                }
            }

            Application.Idle += Application_Idle;
            FormClosed       += Form_FormClosed;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region ø̼  óϱ - Application_Idle(sender, e)

        /// <summary>
        /// ø̼  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Application_Idle(object sender, EventArgs e)
        {
            Image<Bgr, Byte> captureImage = this.capture.QueryFrame();

            Image<Gray, Byte> grayscaleImage = captureImage.Convert<Gray, Byte>();

            grayscaleImage._EqualizeHist();

            Rectangle imageRectangle = grayscaleImage.ROI;

            Rectangle mouseStableRectangle = new Rectangle
            (
                (int)(imageRectangle.Width  * 0.4),
                (int)(imageRectangle.Height * 0.4),
                (int)(imageRectangle.Width  * 0.2),
                (int)(imageRectangle.Height * 0.2)
            );

            captureImage.Draw(mouseStableRectangle, new Bgr(255, 0, 0), 1);

            Rectangle[] faceRectangleArray = this.classifier.DetectMultiScale
            (
                grayscaleImage,
                1.1,
                10,
                new Size(20, 20),
                Size.Empty
            );

            if(faceRectangleArray.Length > 0)
            {
                Rectangle maximumFaceRectangle = faceRectangleArray[0];

                for(int i = 1; i < faceRectangleArray.Length; i++)
                {
                    if(faceRectangleArray[i].Width * faceRectangleArray[i].Height > maximumFaceRectangle.Width * maximumFaceRectangle.Height)
                    {
                        maximumFaceRectangle = faceRectangleArray[i];
                    }
                }

                captureImage.Draw(maximumFaceRectangle, new Bgr(255, 255, 0.0), 1);

                Point maximumFaceRectangleCenter = new Point
                (
                    maximumFaceRectangle.X + maximumFaceRectangle.Width  / 2,
                    maximumFaceRectangle.Y + maximumFaceRectangle.Height / 2
                );

                Point imageRectangleCenter = new Point
                (
                    imageRectangle.X + imageRectangle.Width  / 2,
                    imageRectangle.Y + imageRectangle.Height / 2
                );

                captureImage.Draw
                (
                    new Cross2DF(maximumFaceRectangleCenter, maximumFaceRectangle.Width * 0.1f, maximumFaceRectangle.Height * 0.1f),
                    new Bgr(0, 255, 0),
                    1
                );

                if(!mouseStableRectangle.Contains(maximumFaceRectangleCenter))
                {
                    double horizontalFraction = (double)(maximumFaceRectangleCenter.X - imageRectangleCenter.X) / imageRectangle.Width;
                    double verticalFraction   = (double)(maximumFaceRectangleCenter.Y - imageRectangleCenter.Y) / imageRectangle.Height;

                    Rectangle primaryScreenRectangle = Screen.PrimaryScreen.Bounds;

                    int maximumMouseSpeed = primaryScreenRectangle.Width / 20;

                    Point mousePoint;

                    GetCursorPos(out mousePoint);

                    mousePoint.X = Math.Min(Math.Max(0, mousePoint.X + (int)((maximumMouseSpeed / 2) * horizontalFraction)), primaryScreenRectangle.Width );
                    mousePoint.Y = Math.Min(Math.Max(0, mousePoint.Y + (int)((maximumMouseSpeed / 2) * verticalFraction  )), primaryScreenRectangle.Height);
               
                    SetCursorPos(mousePoint.X, mousePoint.Y);
                }
            }

            this.imageBox.Image = captureImage;
        }

        #endregion
        #region  ݴ  óϱ - Form_FormClosed(sender, e)

        /// <summary>
        ///  ݴ  óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void Form_FormClosed(object sender, FormClosedEventArgs e)
        {
            if(this.capture != null)
            {
                this.capture.Dispose();
            }
        }

        #endregion
        #region   ư Ŭ óϱ - flipHorizontalButton_Click(sender, e)

        /// <summary>
        ///   ư Ŭ óϱ
        /// </summary>
        /// <param name="sender">̺Ʈ ߻</param>
        /// <param name="e">̺Ʈ </param>
        private void flipHorizontalButton_Click(object sender, EventArgs e)
        {
            if(this.capture != null)
            {
                this.capture.FlipHorizontal = !this.capture.FlipHorizontal;
            }
        }

        #endregion
    }
}